function loglik_true = compute_ordinal_loglik(true_z, mu, lambda, b_lower, b_upper)
% true_z is an ordinal value (say 1,2,...,K)
% b_lower and b_upper are K x 1 vectors specifying the lower and upper thresholds
% mu, lambda are the parameters of the gaussian posterior z_m
% code computes p(true_z) = difference of gaussian cdfs
% example usage: compute_ordinal_loglik([1;2], [0;0], [1;1], [-inf;0], [0;inf])

invsigma = lambda .^ 0.5;
lower_normalized = (b_lower(true_z) - mu) .* invsigma;
upper_normalized = (b_upper(true_z) - mu) .* invsigma;
tmp = normcdf(upper_normalized) - normcdf(lower_normalized);
assert(sum(tmp<0)==0);
tmp(tmp==0) = 1e-100;
loglik_true = log(tmp);