function [predZ] = regression_ml(settings, x, nQueryUrls, nLabelers, nLabels, K, ...
    v, init_id, difficulty, expt)
% EM for Continuous-ML model
% ML updates for tau can become unstable

% TODO: modify code to work with mapped v values, add regularization for tau

RAND_INIT = 1;

% caching some computations
rnm2 = x(:,3).^2;
numratings_n = accumarray(x(:,2),ones(nLabels,1));
numratings_m = accumarray(x(:,1),ones(nLabels,1));
idx_m_nz = find(numratings_m>0);
idx_n_nz = find(numratings_n>0);

assert(strcmp(difficulty, 'off') == 1);

% parameter initializations
if RAND_INIT
    tau = 2*rand(nLabelers,1);  % expected precision = 1
else
    tau = ones(nLabelers,1);
end
mu = mean(v) * ones(nQueryUrls,1);  % this value doesn't matter: mu(idx_m_nz) will be updated in the E-step
lambda_prior = 0;
mu_prior = mean(v);

cost_function_M = -Inf*ones(settings.maxIter,1);
cost_function_old = -Inf;

for iter = 1:settings.maxIter
    
    % update mu
    tmp_den = lambda_prior + accumarray(x(:,1), tau(x(:,2)), [nQueryUrls 1]);
    tmp_num  = mu_prior*lambda_prior + accumarray(x(:,1), tau(x(:,2)).*x(:,3), [nQueryUrls 1]);
    mu(idx_m_nz) = tmp_num(idx_m_nz) ./ tmp_den(idx_m_nz);
    
    % update tau
    tmp_var_r = max(0, rnm2 - 2*x(:,3).*mu(x(:,1)) + mu(x(:,1)).^2);
    tmp = accumarray(x(:,2), tmp_var_r, [nLabelers, 1]);
    
    tau = numratings_n ./ tmp; % can blow up to infinity
    cost_function_M(iter) = 0.5 * ( log(tau(idx_n_nz))' * numratings_n(idx_n_nz) ...
        + sum(tau(x(:,2)) .* (tmp_var_r + settings.EPS)) ); % to prevent nan when tau tends to infinity
    if settings.verbose >= 2
        fprintf('iter = %4d, Q = %f, range(tau)=(%.2f,%.2f)\n',iter, cost_function_M(iter), min(tau), max(tau));
    end
    assert(cost_function_M(iter) >= cost_function_old)
    
    if (abs(cost_function_M(iter)-cost_function_old) < settings.thresh)
        fprintf('Breaking out since del(cost function) < %f\n',settings.thresh);
        break;
    end
    cost_function_old  = cost_function_M(iter);
end

predZ = mu;
varZ = zeros(size(mu));

if settings.write
    dlmwrite([settings.op_dir expt '.' 'regression_ml' '.difficulty' difficulty '.init_id' num2str(init_id) '.predGndTruth'],...
        [(1:nQueryUrls)' predZ varZ], 'delimiter', '\t', 'precision', 10);
    
    save([settings.op_dir expt '.' 'regression_ml' '.difficulty' difficulty '.init_id' num2str(init_id) '.mat'] ...
        ,'predZ','varZ', 'mu', 'tau', 'cost_function_M');
end
